/*
 * This software is the confidential and proprietary information of
 * Smardec. You shall not disclose or distribute such Confidential Information.
 * This software shall not be integrated, copied (partially or fully),
 * or reverse engineered into any other software, program, or code that is
 * not intended for Smardec. You shall use it only in accordance with the terms
 * of the license agreement you entered into with Smardec.
 *
 * Copyright (C) 2004-2006 Smardec. All rights reserved.
 *
 * http://www.smardec.com
 *
 */

package com.smardec.demo.j2native.linux.usermanager.info;

import com.smardec.demo.j2native.linux.usermanager.UserManager;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.util.Arrays;
import java.util.Comparator;


public class UIGroupPropertiesDialog extends JDialog {
    private GroupInfo groupInfo;

    private JTextField userNameField;

    private JList usersSelectionList;

    private boolean root;

    public UIGroupPropertiesDialog(Frame parentFrame, GroupInfo groupInfo, boolean root) {
        super(parentFrame, "Group Properties: <" + groupInfo.getIdent() + ">", true);

        this.root = root;
        this.groupInfo = groupInfo;

        Container cp = this.getContentPane();
        cp.setLayout(new BorderLayout());

        JPanel mainWindowPanel = new JPanel(new GridBagLayout(), true);

        JTabbedPane tabbedPane = new JTabbedPane(JTabbedPane.TOP);
        JPanel userDataTab = createUserDataTab();
        JPanel groupsTab = createUsersTab();

        tabbedPane.addTab("Group Data", new JScrollPane(userDataTab));
        tabbedPane.addTab("Group Users", groupsTab);
        constrain(mainWindowPanel, tabbedPane, 0, 0, 2, 1, GridBagConstraints.BOTH, GridBagConstraints.CENTER, 1.0, 1.0, 5, 5, 5, 5);

        JButton okBtn = new JButton("Ok");
        okBtn.setEnabled(isRoot());
        okBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                System.out.println("JButton: \"Ok\"");
            }
        });
//    applyFilterBtn.setSize(applyFilterBtn.getWidth(), filterField.getHeight());
        constrain(mainWindowPanel, okBtn, 0, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST, 1.0, 0.0, 5, 5, 5, 5);

        JButton cancelBtn = new JButton("Cancel");
        cancelBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                UIGroupPropertiesDialog.this.dispose();
            }
        });
//    applyFilterBtn.setSize(applyFilterBtn.getWidth(), filterField.getHeight());
        constrain(mainWindowPanel, cancelBtn, 1, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST, 0.0, 0.0, 5, 5, 5, 5);

        cp.add(mainWindowPanel, BorderLayout.CENTER);
        pack();

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        int frameWidth = 410;
        int frameHeight = 300;
        setBounds((screenSize.width - frameWidth) / 2, (screenSize.height - frameHeight) / 2, frameWidth, frameHeight);
        // init values
        if (groupInfo == null) {
            return;
        }
        userNameField.setText(groupInfo.getGroupName());

        DefaultListModel userList = new DefaultListModel();
        UserInfo[] userInfos = UserManager.getInstance().getUserInfos();
        Arrays.sort(userInfos, new Comparator() {
            public int compare(Object o1, Object o2) {
                UserInfo user1 = (UserInfo) o1;
                UserInfo user2 = (UserInfo) o2;
                return user1.getUserName().compareTo(user2.getUserName());
            }
        });
        java.util.List groupUsersList = groupInfo.getGroupMembers();
        for (int i = 0; i < userInfos.length; i++) {
            UserInfo userInfo = userInfos[i];
            boolean selected = false;
            String userName = userInfo.getUserName();
            for (int j = groupUsersList.size() - 1; j >= 0; j--) {
                if (userName.equals(groupUsersList.get(j))) {
                    selected = true;
                    break;
                }
            }
            JCheckBox checkBox = new JCheckBox(userInfo.getUserName());
            checkBox.setSelected(selected);
            userList.addElement(checkBox);
        }
        usersSelectionList.setModel(userList);
    }

    public GroupInfo getGroupInfo() {
        return groupInfo;
    }

    private JPanel createUserDataTab() {
        JPanel panel = new JPanel(new GridBagLayout(), true);

        JLabel unLabel = new JLabel("User Name:");
//        unLabel.setEnabled(isRoot());
        unLabel.setForeground(Color.black);
        constrain(panel, unLabel, 0, 0, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, 0.0, 0.0, 5, 5, 5, 5);

        userNameField = new JTextField();
        userNameField.setForeground(Color.black);
        userNameField.setEditable(isRoot());
        constrain(panel, userNameField, 1, 0, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1.0, 0.0, 5, 5, 5, 5);

        JLabel fillLabel = new JLabel();
        constrain(panel, fillLabel, 0, 1, 2, 1, GridBagConstraints.BOTH, GridBagConstraints.CENTER, 1.0, 1.0, 5, 5, 5, 5);

        return panel;
    }


    private JPanel createUsersTab() {
        JPanel panel = new JPanel(new GridBagLayout(), true);

        JLabel iLabel = new JLabel("Select the users to join this group:");
//        iLabel.setEnabled(isRoot());
        iLabel.setForeground(Color.black);
        constrain(panel, iLabel, 0, 0, 2, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST, 1.0, 0.0, 5, 5, 5, 5);

        Object[] objects = new Object[10];
        for (int i = 0; i < objects.length; i++) {
            objects[i] = new JCheckBox("User #" + i);
        }
        usersSelectionList = new JList(objects) {
            private Object prevSelected;

            protected void processKeyEvent(KeyEvent e) {
                super.processKeyEvent(e);
                JCheckBox checkBox = (JCheckBox) getSelectedValue();
                if ((checkBox != null) && (e.getKeyCode() == KeyEvent.VK_SPACE) && (e.getID() == KeyEvent.KEY_PRESSED)) {
                    checkBox.setSelected(!checkBox.isSelected());
                    repaint();
                }
            }

            protected void processMouseEvent(MouseEvent e) {
                int id = e.getID();
                switch (id) {
                    case MouseEvent.MOUSE_PRESSED:
                        prevSelected = getSelectedValue();
                        super.processMouseEvent(e);
                        break;
                    case MouseEvent.MOUSE_CLICKED:
                        super.processMouseEvent(e);
                        if ((prevSelected != null) && (prevSelected == getSelectedValue()) && (MouseEvent.MOUSE_CLICKED == e.getID())) {
                            Rectangle bounds = getCellBounds(getSelectedIndex(), getSelectedIndex());
                            if ((e.getY() >= bounds.y) && (e.getY() <= bounds.y + bounds.height)) {
                                JCheckBox checkBox = (JCheckBox) getSelectedValue();
                                checkBox.setSelected(!checkBox.isSelected());
                                repaint();
                            }
                        }
                        break;
                    default:
                        super.processMouseEvent(e);
                        break;
                }


            }
        };
        usersSelectionList.setCellRenderer(new ListCellRenderer() {
            public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
                JComponent component = (JComponent) value;
                component.setBackground((isSelected) ? list.getSelectionBackground() : list.getBackground());
                if (isRoot()) {
                    component.setEnabled(true);
                } else {
                    component.setEnabled(false);
                }
                return component;
            }
        });
        usersSelectionList.setEnabled(isRoot());
        constrain(panel, new JScrollPane(usersSelectionList), 0, 1, 2, 1, GridBagConstraints.BOTH, GridBagConstraints.WEST, 1.0, 1.0, 5, 5, 5, 5);

        return panel;
    }

    private boolean isRoot() {
        return root;
    }

    private static void constrain(Container container, Component component, int x, int y, int width, int height, int fill, int anchor, double weightx, double weighty, int insetTop, int insetLeft, int insetBottom, int insetRight) {
        Insets insets = new Insets(insetTop, insetLeft, insetBottom, insetRight);
        GridBagConstraints gbc = new GridBagConstraints(x, y, width, height, weightx, weighty, anchor, fill, insets, 0, 0);
        LayoutManager lm = container.getLayout();
        GridBagLayout gbl = (GridBagLayout) lm;
        gbl.setConstraints(component, gbc);
        container.add(component);
    }
}
